from braintree.resource import Resource

class ConnectedMerchantStatusTransitioned(Resource):

    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

    @property
    def merchant_id(self):
        return self.merchant_public_id



from braintree.error_result import ErrorResult
from braintree.merchant_account import MerchantAccount
from braintree.paginated_collection import PaginatedCollection
from braintree.paginated_result import PaginatedResult
from braintree.resource import Resource
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.exceptions.not_found_error import NotFoundError

class MerchantAccountGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def create(self, params={}):
        Resource.verify_keys(params, MerchantAccountGateway._detect_signature(params))
        return self._post("/merchant_accounts/create_via_api", {"merchant_account": params})

    def update(self, merchant_account_id, params={}):
        Resource.verify_keys(params, MerchantAccountGateway._update_signature())
        return self._put("/merchant_accounts/%s/update_via_api" % merchant_account_id, {"merchant_account": params})

    def find(self, merchant_account_id):
        try:
            if merchant_account_id is None or merchant_account_id.strip() == "":
                raise NotFoundError()
            response = self.config.http().get(self.config.base_merchant_path() + "/merchant_accounts/" + merchant_account_id)
            return MerchantAccount(self.gateway, response["merchant_account"])
        except NotFoundError:
            raise NotFoundError("merchant account with id " + repr(merchant_account_id) + " not found")

    def create_for_currency(self, params={}):
        return self._post("/merchant_accounts/create_for_currency", {"merchant_account": params})

    def all(self):
        pc = PaginatedCollection(self._fetch_merchant_accounts)
        return SuccessfulResult({"merchant_accounts": pc})

    def _fetch_merchant_accounts(self, current_page):
        response = self.config.http().get(self.config.base_merchant_path() + "/merchant_accounts/?page=" + str(current_page))
        body = response["merchant_accounts"]
        merchant_accounts = [MerchantAccount(self.gateway, merchant_account) for merchant_account in ResourceCollection._extract_as_array(body, "merchant_account")]
        return PaginatedResult(body["total_items"], body["page_size"], merchant_accounts)

    def _post(self, url, params={}):
        response = self.config.http().post(self.config.base_merchant_path() + url, params)

        if "response" in response:
            response = response["response"]

        if "merchant_account" in response:
            return SuccessfulResult({"merchant_account": MerchantAccount(self.gateway, response["merchant_account"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def _put(self, url, params={}):
        response = self.config.http().put(self.config.base_merchant_path() + url, params)
        if "merchant_account" in response:
            return SuccessfulResult({"merchant_account": MerchantAccount(self.gateway, response["merchant_account"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    @staticmethod
    def _detect_signature(attributes):
        if 'applicant_details' in attributes:
            # Warn deprecated
            return MerchantAccountGateway._create_deprecated_signature()
        else:
            return MerchantAccountGateway._create_signature()

    @staticmethod
    def _create_deprecated_signature():
        return [
            {'applicant_details': [
                'company_name',
                'first_name',
                'last_name',
                'email',
                'phone',
                'date_of_birth',
                'ssn',
                'tax_id',
                'routing_number',
                'account_number',
                {'address': [
                    'street_address',
                    'postal_code',
                    'locality',
                    'region']}
                ]
            },
            'tos_accepted',
            'master_merchant_account_id',
            'id'
        ]

    @staticmethod
    def _create_signature():
        return [
            {'individual': [
                'first_name',
                'last_name',
                'email',
                'phone',
                'date_of_birth',
                'ssn',
                {'address': [
                    'street_address',
                    'postal_code',
                    'locality',
                    'region']}
                ]
            },
            {'business': [
                'dba_name',
                'legal_name',
                'tax_id',
                {'address': [
                    'street_address',
                    'postal_code',
                    'locality',
                    'region']}
                ]
            },
            {'funding': [
                'routing_number',
                'account_number',
                'destination',
                'email',
                'mobile_phone',
                'descriptor',
                ]
            },
            'tos_accepted',
            'master_merchant_account_id',
            'id'
        ]

    @staticmethod
    def _update_signature():
        return [
            {'individual': [
                'first_name',
                'last_name',
                'email',
                'phone',
                'date_of_birth',
                'ssn',
                {'address': [
                    'street_address',
                    'postal_code',
                    'locality',
                    'region']}
                ]
            },
            {'business': [
                'dba_name',
                'legal_name',
                'tax_id',
                {'address': [
                    'street_address',
                    'postal_code',
                    'locality',
                    'region']}
                ]
            },
            {'funding': [
                'routing_number',
                'account_number',
                'destination',
                'email',
                'mobile_phone',
                'descriptor',
                ]
            },
            'master_merchant_account_id',
            'id'
        ]



import braintree
import re
import warnings
from braintree.dispute import Dispute
from braintree.dispute_details import DisputeEvidence
from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult
from braintree.exceptions.not_found_error import NotFoundError
from braintree.paginated_result import PaginatedResult
from braintree.paginated_collection import PaginatedCollection
from braintree.resource_collection import ResourceCollection

class DisputeGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def accept(self, dispute_id):
        try:
            if dispute_id is None or dispute_id.strip() == "":
                raise NotFoundError()

            response = self.config.http().put(self.config.base_merchant_path() + "/disputes/" + dispute_id + "/accept")

            if "api_error_response" in response:
                return ErrorResult(self.gateway, response["api_error_response"])
            else:
                return SuccessfulResult()
        except NotFoundError:
            raise NotFoundError("dispute with id " + repr(dispute_id) + " not found")

    def add_file_evidence(self, dispute_id, document_upload_id_or_request):
        request = document_upload_id_or_request if isinstance(document_upload_id_or_request, dict) else { "document_id": document_upload_id_or_request }

        try:
            if dispute_id is None or dispute_id.strip() == "":
                raise NotFoundError()

            if request.get("category") is not None and not isinstance(request["category"], str):
                raise ValueError("category must be a string")

            if request.get("document_id") is None or request["document_id"].strip() == "":
                raise ValueError("document_id cannot be blank")

            response = self.config.http().post(self.config.base_merchant_path() + "/disputes/" + dispute_id + "/evidence", {
                "evidence": {
                    "document_upload_id": request.get("document_id"),
                    "category": request.get("category")
                    }
            })

            if "evidence" in response:
                return SuccessfulResult({
                    "evidence": DisputeEvidence(response["evidence"])
                })
            elif "api_error_response" in response:
                return ErrorResult(self.gateway, response["api_error_response"])

        except NotFoundError:
            raise NotFoundError("dispute with id " + repr(dispute_id) + " not found")

    def add_text_evidence(self, dispute_id, content_or_request):
        request = content_or_request if isinstance(content_or_request, dict) else { "content": content_or_request }

        if dispute_id is None or dispute_id.strip() == "":
            raise NotFoundError("dispute_id cannot be blank")
        if request.get("content") is None or request["content"].strip() == "":
            raise ValueError("content cannot be blank")

        try:
            if request.get("sequence_number") is not None:
                request["sequence_number"] = int(request["sequence_number"])
        except ValueError:
            raise ValueError("sequence_number must be an integer")

        category = request.get("category", request.get("tag"))

        if "tag" in request.keys():
            warnings.warn("Please use category instead", DeprecationWarning)

        if category is not None and not isinstance(category, str):
            raise ValueError("category must be a string")

        try:
            response = self.config.http().post(self.config.base_merchant_path() + "/disputes/" + dispute_id + "/evidence", {
                "evidence": {
                    "comments": request.get("content"),
                    "category": category,
                    "sequence_number": request.get("sequence_number")
                }
            })

            if "evidence" in response:
                return SuccessfulResult({
                    "evidence": DisputeEvidence(response["evidence"])
                })
            elif "api_error_response" in response:
                return ErrorResult(self.gateway, response["api_error_response"])
        except NotFoundError:
            raise NotFoundError("Dispute with ID " + repr(dispute_id) + " not found")

    def finalize(self, dispute_id):
        try:
            if dispute_id is None or dispute_id.strip() == "":
                raise NotFoundError()

            response = self.config.http().put(self.config.base_merchant_path() + "/disputes/" + dispute_id + "/finalize")

            if "api_error_response" in response:
                return ErrorResult(self.gateway, response["api_error_response"])
            else:
                return SuccessfulResult()
        except NotFoundError:
            raise NotFoundError("dispute with id " + repr(dispute_id) + " not found")

    def find(self, dispute_id):
        try:
            if dispute_id is None or dispute_id.strip() == "":
                raise NotFoundError()

            response = self.config.http().get(self.config.base_merchant_path() + "/disputes/" + dispute_id)
            return Dispute(response["dispute"])
        except NotFoundError:
            raise NotFoundError("dispute with id " + repr(dispute_id) + " not found")

    def remove_evidence(self, dispute_id, evidence_id):
        try:
            if dispute_id is None or dispute_id.strip() == "":
                raise NotFoundError()
            if evidence_id is None or evidence_id.strip() == "":
                raise NotFoundError()

            response = self.config.http().delete(self.config.base_merchant_path() + "/disputes/" + dispute_id + "/evidence/" + evidence_id)

            if "api_error_response" in response:
                return ErrorResult(self.gateway, response["api_error_response"])
            else:
                return SuccessfulResult()
        except NotFoundError:
            raise NotFoundError("evidence with id " + repr(evidence_id) + " for dispute with id " + repr(dispute_id) + " not found")

    def search(self, *query):
        if isinstance(query[0], list):
            query = query[0]

        self.search_criteria = self.__criteria(query)

        pc = PaginatedCollection(self.__fetch_disputes)
        return SuccessfulResult({"disputes": pc})

    def __fetch_disputes(self, page):
        response = self.config.http().post(self.config.base_merchant_path() + "/disputes/advanced_search?page=" + str(page), {"search": self.search_criteria})
        body = response["disputes"]

        disputes = [Dispute(item) for item in ResourceCollection._extract_as_array(response["disputes"], "dispute")]
        return PaginatedResult(body["total_items"], body["page_size"], disputes)

    def __criteria(self, query):
        criteria = {}

        for term in query:
            if criteria.get(term.name):
                criteria[term.name] = dict(list(criteria[term.name].items()) + list(term.to_param().items()))
            else:
                criteria[term.name] = term.to_param()
        return criteria



class Search:
    """
    Collection of classes used to build search queries.

    Each Builder class defines one or more methods that returns a Node
    object with the name of the field, the comparator, and the value.
    """
    class IsNodeBuilder(object):
        """Builds a query for value equality."""
        def __init__(self, name):
            self.name = name

        def __eq__(self, value):
            return self.is_equal(value)

        def is_equal(self, value):
            return Search.Node(self.name, {"is": value})

    class EqualityNodeBuilder(IsNodeBuilder):
        """Builds a query for value inequality."""
        def __ne__(self, value):
            return self.is_not_equal(value)

        def is_not_equal(self, value):
            return Search.Node(self.name, {"is_not": value})

    class KeyValueNodeBuilder(object):
        """Builds a query based on a key-value map."""
        def __init__(self, name):
            self.name = name

        def __eq__(self, value):
            return self.is_equal(value)

        def is_equal(self, value):
            return Search.Node(self.name, value)

        def __ne__(self, value):
            return self.is_not_equal(value)

        def is_not_equal(self, value):
            return Search.Node(self.name, not value)

    class PartialMatchNodeBuilder(EqualityNodeBuilder):
        """Builds a query for matching parts of a sequence."""
        def starts_with(self, value):
            return Search.Node(self.name, {"starts_with": value})

        def ends_with(self, value):
            return Search.Node(self.name, {"ends_with": value})

    class EndsWithNodeBuilder(object):
        def __init__(self, name):
            self.name = name

        def ends_with(self, value):
            return Search.Node(self.name, {"ends_with": value})

    class TextNodeBuilder(PartialMatchNodeBuilder):
        """Builds a query for matching any part of a sequence."""
        def contains(self, value):
            return Search.Node(self.name, {"contains": value})

    class Node(object):
        """Container for part of a search query."""
        def __init__(self, name, dict):
            self.name = name
            self.dict = dict

        def to_param(self):
            return self.dict

    class MultipleValueNodeBuilder(object):
        """Builds a query to check membership in a sequence."""
        def __init__(self, name, whitelist = []):
            self.name = name
            self.whitelist = whitelist

        def in_list(self, *values):
            if isinstance(values[0], list):
                values = values[0]

            invalid_args = set(values) - set(self.whitelist)
            if len(self.whitelist) > 0 and len(invalid_args) > 0:
                error_string = "Invalid argument(s) for %s: %s" % (self.name, ", ".join(invalid_args))
                raise AttributeError(error_string)
            return Search.Node(self.name, list(values))

        def __eq__(self, value):
            return self.in_list([value])

    class MultipleValueOrTextNodeBuilder(TextNodeBuilder, MultipleValueNodeBuilder):
        """Builder node supporting contains and in_list."""
        def __init__(self, name, whitelist = []):
            Search.MultipleValueNodeBuilder.__init__(self, name, whitelist)

    class RangeNodeBuilder(object):
        """Builds a query supporting <=, >=, or == value."""
        def __init__(self, name):
            self.name = name

        def __eq__(self, value):
            return self.is_equal(value)

        def is_equal(self, value):
            return Search.EqualityNodeBuilder(self.name) == value

        def __ge__(self, min):
            return self.greater_than_or_equal_to(min)

        def greater_than_or_equal_to(self, min):
            return Search.Node(self.name, {"min": min})

        def __le__(self, max):
            return self.less_than_or_equal_to(max)

        def less_than_or_equal_to(self, max):
            return Search.Node(self.name, {"max": max})

        def between(self, min, max):
            return Search.Node(self.name, {"min": min, "max": max})



from braintree.configuration import Configuration
from braintree.resource import Resource

class AccountUpdaterDailyReport(Resource):

    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)
        if "report_url" in attributes:
            self.report_url = attributes.pop("report_url")
        if "report_date" in attributes:
            self.report_date = attributes.pop("report_date")

    def __repr__(self):
        detail_list = ["report_url", "report_date"]
        return super(AccountUpdaterDailyReport, self).__repr__(detail_list)



from braintree.resource import Resource
from braintree.configuration import Configuration
from braintree.subscription import Subscription
from braintree.merchant_account import MerchantAccount
from braintree.transaction import Transaction
from braintree.partner_merchant import PartnerMerchant
from braintree.oauth_access_revocation import OAuthAccessRevocation
from braintree.disbursement import Disbursement
from braintree.dispute import Dispute
from braintree.account_updater_daily_report import AccountUpdaterDailyReport
from braintree.error_result import ErrorResult
from braintree.validation_error_collection import ValidationErrorCollection
from braintree.connected_merchant_paypal_status_changed import ConnectedMerchantPayPalStatusChanged
from braintree.connected_merchant_status_transitioned import ConnectedMerchantStatusTransitioned
# NEXT_MAJOR_VERSION Remove this class as legacy Ideal has been removed/disabled in the Braintree Gateway
# DEPRECATED If you're looking to accept iDEAL as a payment method contact accounts@braintreepayments.com for a solution.
from braintree.ideal_payment import IdealPayment
from braintree.granted_payment_instrument_update import GrantedPaymentInstrumentUpdate
from braintree.revoked_payment_method_metadata import RevokedPaymentMethodMetadata
from braintree.local_payment_completed import LocalPaymentCompleted

class WebhookNotification(Resource):
    class Kind(object):
        AccountUpdaterDailyReport = "account_updater_daily_report"
        Check = "check"
        ConnectedMerchantStatusTransitioned = "connected_merchant_status_transitioned"
        ConnectedMerchantPayPalStatusChanged = "connected_merchant_paypal_status_changed"
        PartnerMerchantConnected = "partner_merchant_connected"
        PartnerMerchantDisconnected = "partner_merchant_disconnected"
        PartnerMerchantDeclined = "partner_merchant_declined"
        OAuthAccessRevoked = "oauth_access_revoked"
        SubscriptionCanceled = "subscription_canceled"
        SubscriptionChargedSuccessfully = "subscription_charged_successfully"
        SubscriptionChargedUnsuccessfully = "subscription_charged_unsuccessfully"
        SubscriptionExpired = "subscription_expired"
        SubscriptionTrialEnded = "subscription_trial_ended"
        SubscriptionWentActive = "subscription_went_active"
        SubscriptionWentPastDue = "subscription_went_past_due"
        SubMerchantAccountApproved = "sub_merchant_account_approved"
        SubMerchantAccountDeclined = "sub_merchant_account_declined"
        TransactionDisbursed = "transaction_disbursed"
        TransactionSettled = "transaction_settled"
        TransactionSettlementDeclined = "transaction_settlement_declined"
        DisbursementException = "disbursement_exception"
        Disbursement = "disbursement"
        DisputeOpened = "dispute_opened"
        DisputeLost = "dispute_lost"
        DisputeWon = "dispute_won"
        # NEXT_MAJOR_VERSION Remove this class as legacy Ideal has been removed/disabled in the Braintree Gateway
        # DEPRECATED If you're looking to accept iDEAL as a payment method contact accounts@braintreepayments.com for a solution.
        IdealPaymentComplete = "ideal_payment_complete"
        IdealPaymentFailed = "ideal_payment_failed"
        # NEXT_MAJOR_VERSION remove GrantedPaymentInstrumentUpdate. Kind is not sent by Braintree Gateway.
        # Kind will either be GrantorUpdatedGrantedPaymentMethod or RecipientUpdatedGrantedPaymentMethod.
        GrantedPaymentInstrumentUpdate = "granted_payment_instrument_update"
        GrantorUpdatedGrantedPaymentMethod = "grantor_updated_granted_payment_method"
        RecipientUpdatedGrantedPaymentMethod = "recipient_updated_granted_payment_method"
        GrantedPaymentMethodRevoked = "granted_payment_method_revoked"
        PaymentMethodRevokedByCustomer = "payment_method_revoked_by_customer"
        LocalPaymentCompleted = "local_payment_completed"

    @staticmethod
    def parse(signature, payload):
        return Configuration.gateway().webhook_notification.parse(signature, payload)

    @staticmethod
    def verify(challenge):
        return Configuration.gateway().webhook_notification.verify(challenge)

    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        if "source_merchant_id" not in attributes:
            self.source_merchant_id = None

        if "api_error_response" in attributes["subject"]:
            node_wrapper = attributes["subject"]["api_error_response"]
        else:
            node_wrapper = attributes["subject"]

        if "subscription" in node_wrapper:
            self.subscription = Subscription(gateway, node_wrapper['subscription'])
        elif "merchant_account" in node_wrapper:
            self.merchant_account = MerchantAccount(gateway, node_wrapper['merchant_account'])
        elif "transaction" in node_wrapper:
            self.transaction = Transaction(gateway, node_wrapper['transaction'])
        elif "connected_merchant_status_transitioned" in node_wrapper:
            self.connected_merchant_status_transitioned = ConnectedMerchantStatusTransitioned(gateway, node_wrapper['connected_merchant_status_transitioned'])
        elif "connected_merchant_paypal_status_changed" in node_wrapper:
            self.connected_merchant_paypal_status_changed = ConnectedMerchantPayPalStatusChanged(gateway, node_wrapper['connected_merchant_paypal_status_changed'])
        elif "partner_merchant" in node_wrapper:
            self.partner_merchant = PartnerMerchant(gateway, node_wrapper['partner_merchant'])
        elif "oauth_application_revocation" in node_wrapper:
            self.oauth_access_revocation = OAuthAccessRevocation(node_wrapper["oauth_application_revocation"])
        elif "disbursement" in node_wrapper:
            self.disbursement = Disbursement(gateway, node_wrapper['disbursement'])
        elif "dispute" in node_wrapper:
            self.dispute = Dispute(node_wrapper['dispute'])
        elif "account_updater_daily_report" in node_wrapper:
            self.account_updater_daily_report = AccountUpdaterDailyReport(gateway, node_wrapper['account_updater_daily_report'])
        # NEXT_MAJOR_VERSION Remove this class as legacy Ideal has been removed/disabled in the Braintree Gateway
        # DEPRECATED If you're looking to accept iDEAL as a payment method contact accounts@braintreepayments.com for a solution.
        elif "ideal_payment" in node_wrapper:
            self.ideal_payment = IdealPayment(gateway, node_wrapper['ideal_payment'])
        elif "granted_payment_instrument_update" in node_wrapper:
            self.granted_payment_instrument_update = GrantedPaymentInstrumentUpdate(gateway, node_wrapper["granted_payment_instrument_update"])
        elif attributes["kind"] in [WebhookNotification.Kind.GrantedPaymentMethodRevoked, WebhookNotification.Kind.PaymentMethodRevokedByCustomer]:
            self.revoked_payment_method_metadata = RevokedPaymentMethodMetadata(gateway, node_wrapper)
        elif "local_payment" in node_wrapper:
            self.local_payment_completed = LocalPaymentCompleted(gateway, node_wrapper["local_payment"])

        if "errors" in node_wrapper:
            self.errors = ValidationErrorCollection(node_wrapper['errors'])
            self.message = node_wrapper['message']



import re
import braintree
from braintree.subscription import Subscription
from braintree.error_result import ErrorResult
from braintree.exceptions.not_found_error import NotFoundError
from braintree.resource import Resource
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.transaction import Transaction

class SubscriptionGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def cancel(self, subscription_id):
        response = self.config.http().put(self.config.base_merchant_path() + "/subscriptions/" + subscription_id + "/cancel")
        if "subscription" in response:
            return SuccessfulResult({"subscription": Subscription(self.gateway, response["subscription"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def create(self, params={}):
        Resource.verify_keys(params, Subscription.create_signature())
        response = self.config.http().post(self.config.base_merchant_path() + "/subscriptions", {"subscription": params})
        if "subscription" in response:
            return SuccessfulResult({"subscription": Subscription(self.gateway, response["subscription"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def find(self, subscription_id):
        try:
            if subscription_id is None or subscription_id.strip() == "":
                raise NotFoundError()
            response = self.config.http().get(self.config.base_merchant_path() + "/subscriptions/" + subscription_id)
            return Subscription(self.gateway, response["subscription"])
        except NotFoundError:
            raise NotFoundError("subscription with id " + repr(subscription_id) + " not found")

    def retry_charge(self, subscription_id, amount=None, submit_for_settlement=False):
        response = self.config.http().post(self.config.base_merchant_path() + "/transactions", {"transaction": {
            "amount": amount,
            "subscription_id": subscription_id,
            "type": Transaction.Type.Sale,
            "options": {"submit_for_settlement": submit_for_settlement}
            }})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def search(self, *query):
        if isinstance(query[0], list):
            query = query[0]

        response = self.config.http().post(self.config.base_merchant_path() + "/subscriptions/advanced_search_ids", {"search": self.__criteria(query)})
        return ResourceCollection(query, response, self.__fetch)

    def update(self, subscription_id, params={}):
        Resource.verify_keys(params, Subscription.update_signature())
        response = self.config.http().put(self.config.base_merchant_path() + "/subscriptions/" + subscription_id, {"subscription": params})
        if "subscription" in response:
            return SuccessfulResult({"subscription": Subscription(self.gateway, response["subscription"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def __criteria(self, query):
        criteria = {}
        for term in query:
            if criteria.get(term.name):
                criteria[term.name] = dict(list(criteria[term.name].items()) + list(term.to_param().items()))
            else:
                criteria[term.name] = term.to_param()
        return criteria

    def __fetch(self, query, ids):
        criteria = self.__criteria(query)
        criteria["ids"] = braintree.subscription_search.SubscriptionSearch.ids.in_list(ids).to_param()
        response = self.config.http().post(self.config.base_merchant_path() + "/subscriptions/advanced_search", {"search": criteria})
        return [Subscription(self.gateway, item) for item in ResourceCollection._extract_as_array(response["subscriptions"], "subscription")]




from braintree.resource import Resource
from braintree.merchant_account import MerchantAccount

class Merchant(Resource):
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        if "merchant_accounts" in attributes:
            self.merchant_accounts = [MerchantAccount(gateway, ma) for ma in attributes.get("merchant_accounts")]



from braintree.attribute_getter import AttributeGetter

class DisputeEvidence(AttributeGetter):
    def __init__(self, attributes):
        attributes["tag"] = attributes.get("category")
        AttributeGetter.__init__(self, attributes)



from braintree.search import Search

class CustomerSearch:
      address_extended_address             = Search.TextNodeBuilder("address_extended_address")
      address_first_name                   = Search.TextNodeBuilder("address_first_name")
      address_last_name                    = Search.TextNodeBuilder("address_last_name")
      address_locality                     = Search.TextNodeBuilder("address_locality")
      address_postal_code                  = Search.TextNodeBuilder("address_postal_code")
      address_region                       = Search.TextNodeBuilder("address_region")
      address_street_address               = Search.TextNodeBuilder("address_street_address")
      address_country_name                 = Search.TextNodeBuilder("address_country_name")
      cardholder_name                      = Search.TextNodeBuilder("cardholder_name")
      company                              = Search.TextNodeBuilder("company")
      created_at                           = Search.RangeNodeBuilder("created_at")
      credit_card_expiration_date          = Search.EqualityNodeBuilder("credit_card_expiration_date")
      credit_card_number                   = Search.TextNodeBuilder("credit_card_number")
      email                                = Search.TextNodeBuilder("email")
      fax                                  = Search.TextNodeBuilder("fax")
      first_name                           = Search.TextNodeBuilder("first_name")
      id                                   = Search.TextNodeBuilder("id")
      ids                                  = Search.MultipleValueNodeBuilder("ids")
      last_name                            = Search.TextNodeBuilder("last_name")
      payment_method_token                 = Search.TextNodeBuilder("payment_method_token")
      payment_method_token_with_duplicates = Search.IsNodeBuilder("payment_method_token_with_duplicates")
      phone                                = Search.TextNodeBuilder("phone")
      website                              = Search.TextNodeBuilder("website")
      paypal_account_email                 = Search.TextNodeBuilder("paypal_account_email")



import braintree
import mimetypes
from braintree.document_upload import DocumentUpload
from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult

class DocumentUploadGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def create(self, params={}):
        Resource.verify_keys(params, DocumentUpload.create_signature())

        if "file" in params and not hasattr(params["file"], "read"):
            raise ValueError("file must be a file handle")

        response = self.config.http().post_multipart(self.config.base_merchant_path() + "/document_uploads", *self.__payload(params))

        if "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])
        else:
            return SuccessfulResult({"document_upload": DocumentUpload(self, response["document_upload"])})

    def __file_name(self, file):
        return file.name.split("/")[-1]

    def __content_type(self, file):
        return mimetypes.guess_type(file.name)[0]

    def __payload(self, params):
        file = params.pop("file", None)
        files = {
            "file": (self.__file_name(file), file, self.__content_type(file))
        }
        params["document_upload[kind]"] = params["kind"]

        return (files, params)



from braintree.attribute_getter import AttributeGetter

class ValidationError(AttributeGetter):
    """
    A validation error returned from the server, with information about the error:

    * **attribute**: The field which had an error.
    * **code**: A numeric error code. See :class:`ErrorCodes <braintree.error_codes.ErrorCodes>`
    * **message**: A description of the error.  Note: error messages may change, but the code will not.
    """
    pass



from braintree.us_bank_account import UsBankAccount
from braintree.us_bank_account_verification import UsBankAccountVerification
from braintree.search import Search
from braintree.util import Constants

class UsBankAccountVerificationSearch:
    # Text fields
    account_holder_name = Search.TextNodeBuilder("account_holder_name")
    customer_email = Search.TextNodeBuilder("customer_email")
    customer_id = Search.TextNodeBuilder("customer_id")
    id = Search.TextNodeBuilder("id")
    payment_method_token = Search.TextNodeBuilder("payment_method_token")
    routing_number = Search.TextNodeBuilder("routing_number")

    # Multiple value fields
    ids = Search.MultipleValueNodeBuilder("ids")
    status = Search.MultipleValueNodeBuilder(
        "status",
        Constants.get_all_constant_values_from_class(UsBankAccountVerification.Status)
    )
    verification_method = Search.MultipleValueNodeBuilder(
        "verification_method",
        Constants.get_all_constant_values_from_class(UsBankAccountVerification.VerificationMethod)
    )

    # Range fields
    created_at = Search.RangeNodeBuilder("created_at")

    # Equality fields
    account_type = Search.EqualityNodeBuilder("account_type")

    # Ends-with fieds
    account_number = Search.EndsWithNodeBuilder("account_number")



import braintree
from braintree.resource import Resource
from braintree.configuration import Configuration
from braintree.three_d_secure_info import ThreeDSecureInfo
from braintree.bin_data import BinData

class PaymentMethodNonce(Resource):
    @staticmethod
    def create(payment_method_token):
        return Configuration.gateway().payment_method_nonce.create(payment_method_token)

    @staticmethod
    def find(payment_method_nonce):
        return Configuration.gateway().payment_method_nonce.find(payment_method_nonce)

    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        if "three_d_secure_info" in attributes and not attributes["three_d_secure_info"] is None:
            self.three_d_secure_info = ThreeDSecureInfo(attributes["three_d_secure_info"])
        else:
            self.three_d_secure_info = None

        if "bin_data" in attributes and not attributes["bin_data"] is None:
            self.bin_data = BinData(attributes["bin_data"])
        else:
            self.bin_data = None



from braintree.exceptions.unexpected_error import UnexpectedError

class TimeoutError(UnexpectedError):
    pass


class ConnectTimeoutError(TimeoutError):
    pass


class ReadTimeoutError(TimeoutError):
    pass



class TransactionAmounts(object):
    """ A class of constants for transaction amounts that will cause different statuses. """

    Authorize = "1000.00"
    Decline = "2000.00"
    HardDecline = "2015.00"
    Fail = "3000.00"



from braintree.configuration import Configuration
from braintree.modification import Modification

class AddOn(Modification):
    @staticmethod
    def all():
        return Configuration.gateway().add_on.all()



from decimal import Decimal
from braintree.resource import Resource

class SubscriptionStatusEvent(Resource):
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        self.balance = Decimal(self.balance)
        self.price = Decimal(self.price)



from braintree.resource import Resource

class OAuthAccessRevocation(Resource):
    """
    A class representing an OAuth access revocation.
    """

    def __init__(self, attributes):
        Resource.__init__(self, None, attributes)
